---------------------------------------------------------------------------------------------------
--
-- Name:		Host 2191 top level module
--
-- Project:		ADSP-2191 Host Port Booting
-- File Name:	host_2191.vhd
--
-- Version:		1.01
-- Date:		24.10.2003
-- Author:		Michael Kuegler
-- Company:		Analog Devices, Inc.
--
-- Target:		Altera MAX7000 EPM7256AETC100-7
-- Platform:	Altera Quartus II 2.2, SP 2
--
---------------------------------------------------------------------------------------------------
--
-- Inputs:		CLOCK
--					System clock for PLD
--				RESET
--					Reset for PLD
--				CS
--					External Port chip select
--				WR
--					External Port write strobe
--				RD
--					External Port read strobe
--				ADDRESS[3..0]
--					External Port address bus
--				HACK
--					Host Port acknowledge
--
-- In/Outputs:	DATA[15..0]
--					External Port data bus
--				HAD[15..0]
--					Host Port multiplexed address/data bus
--
-- Outputs:		ERROR
--					Error signal to Host
--				ACK
--					External Port acknowledge
--				HA16
--					Host Port address line 16
--				HCMS
--					Host Port memory select
--				HCIOMS
--					Host Port IO space select
--				HALE
--					Host Port address latch
--				HWR
--					Host Port write strobe
--				HRD
--					Host Port read strobe
--
-- Description: This project implements the VHDL design described in application note
--				"ADSP-2191 Host Port booting".
--				For additional information see readme.txt.
--
---------------------------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

-- top level entity
ENTITY host_2191 IS
	PORT(
		-- control signals
		CLOCK				: IN	STD_LOGIC;
		RESET				: IN	STD_LOGIC;
		ERROR				: OUT	STD_LOGIC;

		-- External Port signals (Host)
		ADDRESS				: IN	STD_LOGIC_VECTOR(3 DOWNTO 0);
		DATA				: INOUT	STD_LOGIC_VECTOR(15 DOWNTO 0);
		CS					: IN	STD_LOGIC;
		WR					: IN	STD_LOGIC;
		RD					: IN	STD_LOGIC;
		ACK					: OUT	STD_LOGIC;

		-- Host Port signals (Target)
		HACK				: IN	STD_LOGIC;
		HAD					: INOUT	STD_LOGIC_VECTOR(15 DOWNTO 0);
		HA16				: OUT	STD_LOGIC;
		HCMS				: OUT	STD_LOGIC;
		HCIOMS				: OUT	STD_LOGIC;
		HALE				: OUT	STD_LOGIC;
		HWR					: OUT	STD_LOGIC;
		HRD					: OUT	STD_LOGIC
	);
END host_2191;

-- top level architecture
ARCHITECTURE host_2191 OF host_2191 IS

	-- Rising edge detector component instantiation
	COMPONENT rising_edge
		PORT(
			i_clock				: IN STD_LOGIC;
			i_input				: IN STD_LOGIC;
			i_reset				: IN STD_LOGIC;
			o_output			: OUT STD_LOGIC
		);
	END COMPONENT rising_edge;

	-- Watchdog component instantiation
	COMPONENT watchdog
		PORT(
			i_clock				: IN STD_LOGIC;
			i_reset				: IN STD_LOGIC;
			i_start				: IN STD_LOGIC;
			o_error				: OUT STD_LOGIC
		);
	END COMPONENT watchdog;

	-- External Port state machine component instantiation
	COMPONENT ep_sm
		PORT(
			i_clock				: IN STD_LOGIC;
			i_finish_ep_access	: IN STD_LOGIC;
			i_reset				: IN STD_LOGIC;
			i_start				: IN STD_LOGIC;
			o_ack				: OUT STD_LOGIC
		);
	END COMPONENT ep_sm;

	-- Host Port state machine component instantiation
	COMPONENT hp_sm
		PORT(
			i_clock				: IN STD_LOGIC;
			i_hack				: IN STD_LOGIC;
			i_host_cycle		: IN STD_LOGIC;
			i_host_mem_sel		: IN STD_LOGIC;
			i_host_strobe		: IN STD_LOGIC;
			i_reset				: IN STD_LOGIC;
			i_start				: IN STD_LOGIC;
			o_hcms				: OUT STD_LOGIC;
			o_hcioms			: OUT STD_LOGIC;
			o_hrd				: OUT STD_LOGIC;
			o_hwr				: OUT STD_LOGIC;
			o_hale				: OUT STD_LOGIC;
			o_latch_had			: OUT STD_LOGIC;
			o_sm_finished		: OUT STD_LOGIC
		);
	END COMPONENT hp_sm;

	-- Data path component instantiation
	COMPONENT data_path
		PORT(
			i_address			: IN STD_LOGIC_VECTOR(3 DOWNTO 0);
			i_ep_rd				: IN STD_LOGIC;
			i_ep_wr				: IN STD_LOGIC;
			i_hp_wr				: IN STD_LOGIC;
			i_latch_had			: IN STD_LOGIC;
			i_reset				: IN STD_LOGIC;
			o_ha16				: OUT STD_LOGIC;
			o_valid_hp_access	: OUT STD_LOGIC;
			io_data				: INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
			io_had				: INOUT STD_LOGIC_VECTOR(15 DOWNTO 0)
		);
	END COMPONENT data_path;

	-- internal signals
	SIGNAL sm_start			: STD_LOGIC;
	SIGNAL system_reset		: STD_LOGIC;

	SIGNAL latched_hack		: STD_LOGIC;
	SIGNAL latched_cs		: STD_LOGIC;
	SIGNAL latched_wr		: STD_LOGIC;
	SIGNAL latched_rd		: STD_LOGIC;

	SIGNAL cs_or_rd			: STD_LOGIC;
	SIGNAL cs_or_wr			: STD_LOGIC;
	SIGNAL ack_dummy		: STD_LOGIC;
	SIGNAL rd_or_wr			: STD_LOGIC;

	SIGNAL hp_sm_start		: STD_LOGIC;
	SIGNAL hp_sm_reset		: STD_LOGIC;
	SIGNAL valid_hp_access	: STD_LOGIC;
	SIGNAL hp_sm_finished	: STD_LOGIC;
	SIGNAL latch_had		: STD_LOGIC;
	SIGNAL hrd_dummy		: STD_LOGIC;
	SIGNAL hwr_dummy		: STD_LOGIC;

	SIGNAL finish_ep_access	: STD_LOGIC;

	SIGNAL wd_reset			: STD_LOGIC;
	SIGNAL wd_error			: STD_LOGIC;

BEGIN
	-- port mapping for data path
	U1 : data_path
	PORT MAP(
		i_address			=> ADDRESS,
		i_ep_rd				=> cs_or_rd,
		i_ep_wr				=> cs_or_wr,
		i_hp_wr				=> hwr_dummy,
		i_latch_had			=> latch_had,
		i_reset				=> system_reset,
		io_data				=> DATA,
		io_had				=> HAD,
		o_ha16				=> HA16,
		o_valid_hp_access	=> valid_hp_access
	);

	-- port mapping for External Port state machine
	U2 : ep_sm
	PORT MAP(
		i_clock				=> CLOCK,
		i_finish_ep_access	=> finish_ep_access,
		i_reset				=> system_reset,
		i_start				=> sm_start,
		o_ack				=> ack_dummy
	);

	-- port mapping for Host Port state machine
	U3 : hp_sm
	PORT MAP(
		i_clock				=> CLOCK,
		i_hack				=> latched_hack,
		i_host_cycle		=> ADDRESS(0),
		i_host_mem_sel		=> ADDRESS(2),
		i_host_strobe		=> cs_or_wr,
		i_reset				=> hp_sm_reset,
		i_start				=> hp_sm_start,
		o_hcms				=> HCMS,
		o_hcioms			=> HCIOMS,
		o_hrd				=> hrd_dummy,
		o_hwr				=> hwr_dummy,
		o_hale				=> HALE,
		o_latch_had			=> latch_had,
		o_sm_finished		=> hp_sm_finished
	);

	-- port mapping for Watchdog
	U4 : watchdog
	PORT MAP(
		i_clock				=> CLOCK,
		i_reset				=> wd_reset,
		i_start				=> sm_start,
		o_error				=> wd_error
	);

	-- port mapping for rising edge detector
	U5 : rising_edge
	PORT MAP(
		i_clock				=> CLOCK,
		i_input				=> rd_or_wr,
		i_reset				=> system_reset,
		o_output			=> sm_start
	);

	-- connections between instantiated blocks
	hp_sm_start				<= valid_hp_access AND sm_start;
	hp_sm_reset				<= system_reset OR wd_error;
	wd_reset				<= system_reset OR ack_dummy;
	finish_ep_access		<= wd_error OR hp_sm_finished OR (NOT valid_hp_access);
	rd_or_wr				<= NOT(cs_or_rd AND cs_or_wr);
	system_reset			<= NOT RESET;
	cs_or_rd				<= latched_cs OR latched_rd;
	cs_or_wr				<= latched_cs OR latched_wr;
	ACK						<= ack_dummy;
	HRD						<= hrd_dummy;
	HWR						<= hwr_dummy;
	ERROR					<= wd_error;

	-- latch asynchronous inputs to reduce risk of metastability
	input_latching: PROCESS(CLOCK)
	BEGIN
		IF (CLOCK'EVENT AND CLOCK = '1') THEN
			latched_hack	<= HACK;
			latched_cs		<= CS;
			latched_wr		<= WR;
			latched_rd		<= RD;
		END IF;
	END PROCESS input_latching;
END host_2191;
